/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#ifndef VNCVIEWERVNCVIEW_H
#define VNCVIEWERVNCVIEW_H

#include <AknView.h>
#include <EIKSBFRM.H>
#include <BITSTD.H>
#include <hal.h>
#include "VNCViewMsg.h"

#include "VNCSendTextDialog.h"
#include "VNCScaleDialog.h"


// Forward declarations
class CVNCViewerContainer;
class CVNCViewerDocument;

/**
The VNC view 
*/
class CVNCViewerVNCView : public CAknView
	{
public:
	static CVNCViewerVNCView* NewLC(CVNCViewerDocument& aDocument);
	virtual ~CVNCViewerVNCView();
	
	// from CQikViewBase
	TUid Id() const;
	void HandleCommandL(TInt aCommand);
	
	void HandleForegroundEventL(TBool aForeground);
		
	void SizeChanged();
	
	void QuitNow();
	void DestroyEngine(TBool aSwitchView);
	
protected: // from CAknView
	IMPORT_C virtual void DoActivateL(const TVwsViewId& aPrevViewId,
	                                  TUid aCustomMessageId,
	                                  const TDesC8& aCustomMessage);

    /** 
     * View deactivation function intended for overriding by sub classes. 
     * This function is called by @c AknViewDeactivated().
     */
	virtual void DoDeactivate();

	virtual void DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane);

private:
	CVNCViewerVNCView(CVNCViewerDocument& aDocument);
	void ConstructL();

private:
	CVNCViewerDocument& 	iDocument;
	CVNCViewerContainer*	iVNCContainer;
	TBool					iNeedToConnect;
	TBool					iGoBack;
	TBool					iExiting;
	HBufC*					iAboutText;
	};

/**
The VNC container, this is a scrollable window which contains the engine
*/
class CVNCViewerContainer : public CCoeControl, public MVNCViewerEvent, public MSendText, public MScale
	{
public:
	enum TButtonMode
		{
		EButtonLeft,
		EButtonMiddle,
		EButtonRight
		};

	enum TModifierMode
		{
		EModShift = 1,
		EModCtrl = 2,
		EModAlt = 4,
		EModFunc = 8,
		EModAll = 15
		};

	CVNCViewerContainer(CVNCViewerVNCView& aView, CVNCViewerDocument& aDocument);
	void ConstructL(const TRect& aRect);
	virtual ~CVNCViewerContainer();

	// Accessors
	TBool LoggedOn();
	TInt Modifiers();
	TButtonMode ButtonMode();
	
	// Exposed methods
	void ConnectL();
	void SetButtonMode(TButtonMode aButtonMode);
	void SetModifiers(TInt aModifier, TBool aOn);
	void SetCustomScaleL();
	void HandleForegroundEventL(TBool aForeground);
	void SendTextDialogL();
	TBool SendKeyL(TInt aKey);
	void RefreshL();
	void SetArrowKeysControlCursor(TBool aOn);
	TBool ArrowKeysControlCursor() { return iArrowKeypressMovesPointer; };
	void FindCursorL();
	void SetScaleL(TReal aScale);
	TReal Scale() { return iScale; };
	
	// from MVCViewerEvent
	HBufC8* ShowPasswordL();
	void CreateBitmapL(TInt aHandle);
	void PutImage(const TRect& aRect);
	void EngineQuit();
	void ServerMessageL(const TDesC& aMessage);
	void MoveSoftCursorL(TInt aX, TInt aY);	
	RWsSprite* CreatePixmapCursorL(TInt aBitmapHandle, TInt aMaskHandle, TInt aHotpointX, TInt aHotpointY);	
	void WarpPointerL(TInt aCursorX, TInt aCursorY);
	void FreeCursorL();

	// From MSendText	
	void SendTextL(TInt aErr, const TDesC& aText);	

	// From MScale	
	void ScaleL(TInt aCmd, const TReal aScale);	

	const CFbsBitmap* Bitmap() const
	{
		return iMainBitmap;
	};

	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);	
	
	void PostQuit();
	
	TInt CountComponentControls() const;
	CCoeControl* ComponentControl(TInt aIndex) const;
	
	void Draw(const TRect& aRect) const;

private:
	TBool CursorInScreenArea();
	void MimicPointerMoveL(TBool aScroll);
	void MimicPointerDownUpL(TBool aDown);
	void TranslateRemoteToLocalPoint(TPoint& aPoint, TBool aRelative) const;
	void TranslateLocalToRemotePoint(TPoint& aPoint, TBool aRelative) const;
	TRect ScaledRect(const TRect& aRect) const;
	TRect ViewRect() const;
	TSize WindowSize() const;
	
private:
	CVNCViewerVNCView& 			iView;
	CVNCViewerDocument& 		iDocument;	
	TBool						iArrowKeypressMovesPointer;
	TInt						iPointerAcceleration;
	TBool						iCursorLocated;
	CVNCViewMsgProcessor*		iVNCViewMsgProc;
	CFbsBitmap*					iMainBitmap;
	CSendTextActive*			iSendTextActive;
	TButtonMode					iButtonMode;
	TBool						iLoggedOn;
	TBool						iCursorCreated;
	RWsSprite					iCursor;
	CFbsBitmap*					iCursorBitmap;
	CFbsBitmap*					iCursorMask;
	TInt						iModifiers;
	CScaleActive*				iScaleActive;
	TReal						iScale;
	TPoint						iOrigin;			// Stored in server coordinates
	TPoint						iCursorPos;			// Stored in server coordinates
	TBool						iPointerDown;
	TBool						iExtendedKeyboard;
	};

#endif // VNCVIEWERVNCVIEW_H
