/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#include <eikedwin.h>

#include "VNCSendTextDialog.h"
#include "VNCViewer.hrh"
#include <VNCViewer.rsg>
#include <avkon.hrh>
#include <aknquerycontrol.h> 


/**
Creates, constructs, execute and destroys the dialog.

@param aStatus The request status to change
@param aText The text to send
@return CVNCSendTextDialog* The dialog
*/
CVNCSendTextDialog* CVNCSendTextDialog::NewL(TRequestStatus& aStatus, HBufC*& aText)
	{
	CVNCSendTextDialog* self = new (ELeave) CVNCSendTextDialog(aStatus, aText);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
	}

/**
Constructor for the dialog.

@param aStatus The request status to change
@param aText The text to send
*/
CVNCSendTextDialog::CVNCSendTextDialog(TRequestStatus& aStatus, HBufC*& aText) 
	: CAknDialog(), iStatus(aStatus), iText(aText)
	{
	}

/**
Destructor for the dialog
*/
CVNCSendTextDialog::~CVNCSendTextDialog()
	{
	}

/**
2nd stage construction of the dialog.
*/
void CVNCSendTextDialog::ConstructL()
	{
	}

/**
Handles all commands in the dialog.
Called by the UI framework when a command has been issued.
The command Ids are defined in the .hrh file.

@param aCommand The command to be executed
*/
TBool CVNCSendTextDialog::OkToExitL(TInt aButtonId)
	{
	if (aButtonId == EAknSoftkeyOk)
		{
		CAknQueryControl* edwin = static_cast<CAknQueryControl*>(Control(EGeneralQuery));
		iText = HBufC::NewL(edwin->GetTextLength());
		TPtr ptr = iText->Des();
		edwin->GetText(ptr);

		TRequestStatus* status = &iStatus;
		User::RequestComplete(status, (iText->Length() > 0) ? KErrNone : KErrCancel);	
		}
	else
		{
		TRequestStatus* status = &iStatus;
		User::RequestComplete(status, KErrCancel);	
		}
	return ETrue;
	}

/**
Cancel the dialog
*/
void CVNCSendTextDialog::Cancel()
	{
	TRequestStatus* status = &iStatus;
	User::RequestComplete(status, KErrCancel);	
	delete this;	
	}

/**
Prepare the dialog
*/
void CVNCSendTextDialog::PreLayoutDynInitL()
	{
	CAknDialog::PreLayoutDynInitL();

	// Had to do this to force the number of lines to be 5
	CAknQueryControl* edwin = static_cast<CAknQueryControl*>(Control(EGeneralQuery));
	edwin->SetNumberOfEditorLines(5);
	}

//---------------------------------------------------------------
//---------------------------------------------------------------

/**
Constructor for the active object.

@param aSendText The callback object
*/
CSendTextActive* CSendTextActive::NewL(MSendText &aSendText)
	{
	CSendTextActive* self = new(ELeave) CSendTextActive(aSendText);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
	}

/**
Destrutor for the active object.
*/
CSendTextActive::~CSendTextActive()
	{
	Cancel();
	delete iText;
	}

/**
Constructor for the active object.

@param aSendText The callback object
*/
CSendTextActive::CSendTextActive(MSendText &aSendText)
: CActive(EPriorityStandard), iSendText(aSendText)
	{
	CActiveScheduler::Add(this);
	}

/**
2nd phase constructor
*/
void CSendTextActive::ConstructL()
	{
	// Create the dialog
	iStatus = KRequestPending;
	iDialog = CVNCSendTextDialog::NewL(iStatus, iText);
	iDialog->ExecuteLD(R_VNCVIEWER_SEND_TEXT_DIALOG);
	SetActive();	
	}
	
/**
The dialog has completed
*/
void CSendTextActive::RunL()
	{
	iSendText.SendTextL(iStatus.Int(), *iText);
	}
	
/**
The dialog needs to be cancelled
*/
void CSendTextActive::DoCancel()
	{
	iDialog->Cancel();	
	iSendText.SendTextL(KErrCancel, KNullDesC());
	}
	
