/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */


#include <QikCommand.h>
#include <eikedwin.h>
#include <eikseced.h>

#include "VNCPasswordDialog.h"
#include "VNCViewer.hrh"
#include <VNCViewer.rsg>

/**
Creates, constructs, execute and destroys the dialog.

@param aPassword The password string
@return TInt The return code
*/
TInt CVNCPasswordDialog::RunDlgLD(TDes& aPassword)
	{
	CVNCPasswordDialog* self = new (ELeave) CVNCPasswordDialog(aPassword);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self->ExecuteLD(R_VNCVIEWER_PASSWORD_DIALOG);
	}

/**
Constructor for the dialog.

@param aPassword The password string
*/
CVNCPasswordDialog::CVNCPasswordDialog(TDes& aPassword) 
	: iCmdManager(CQikCommandManager::Static(*iCoeEnv)), iPassword(aPassword)
	{
	}

/**
Destructor for the dialog
*/
CVNCPasswordDialog::~CVNCPasswordDialog()
	{
	}

/**
2nd stage construction of the dialog.
*/
void CVNCPasswordDialog::ConstructL()
	{
	}

/**
Handles all commands in the dialog.
Called by the UI framework when a command has been issued.
The command Ids are defined in the .hrh file.

@param aCommand The command to be executed
*/
void CVNCPasswordDialog::HandleCommandL(CQikCommand& aCommand)
	{
	switch(aCommand.Id())
		{
		case EVNCViewerPasswordDialogLoginCmd:
		case EVNCViewerPasswordDialogCancelCmd:
			{
			// Signals that the dialog should close with the command id as value
			const CEikSecretEditor* secretEditor = LocateControlByUniqueHandle<const CEikSecretEditor>(EVNCViewerPasswordDialogSecretEditor);
			secretEditor->GetText(iPassword);

			CloseDialog(aCommand.Id());
			break;
			}
		default:
			{
			CQikSimpleDialog::HandleCommandL(aCommand);
			break;
			}
		}
	}

/**
Get the capabilities of the dialog

@return TCoeInputCapabilities The capabilites
*/
TCoeInputCapabilities CVNCPasswordDialog::InputCapabilities() const
	{
	return TCoeInputCapabilities::EAllText;
	}

/**
Handles an event from an observed control. Used to get event when the edwin 
has been changed. If the edwin contains any text the login command shall be
available.

@param aControl The control
@param aEventType The event type
*/
void CVNCPasswordDialog::HandleControlEventL(CCoeControl *aControl, TCoeEvent aEventType)
	{
	if(aEventType == EEventStateChanged)
		{
		switch(aControl->UniqueHandle())
			{
			case EVNCViewerPasswordDialogSecretEditor:
				{
				// Locate the edwin
				const CEikSecretEditor* secret = LocateControlByUniqueHandle<const CEikSecretEditor>(EVNCViewerPasswordDialogSecretEditor);
            	
            	TBuf<16> buf;
            	secret->GetText(buf);
            	
            	// Sets the login command to available if the edwin contains any text
            	const TBool loginIsAvailable = (buf.Length() != 0);
            	iCmdManager.SetAvailable(*this, EVNCViewerPasswordDialogLoginCmd, loginIsAvailable);
				}
			}
		}
	CQikSimpleDialog::HandleControlEventL(aControl, aEventType);
	}
