/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */


#include <QikCommand.h>
#include <eikedwin.h>

#include "VNCSendTextDialog.h"
#include "VNCViewer.hrh"
#include <VNCViewer.rsg>

/**
Creates, constructs, execute and destroys the dialog.

@param aStatus Request status to complete
@param aText Text to set
@return CVNCSendTextDialog* The dialog
*/
CVNCSendTextDialog* CVNCSendTextDialog::NewL(TRequestStatus& aStatus, HBufC*& aText)
	{
	CVNCSendTextDialog* self = new (ELeave) CVNCSendTextDialog(aStatus, aText);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
	}

/**
Constructor for the dialog.

@param aStatus Request status to complete
@param aText Text to set
*/
CVNCSendTextDialog::CVNCSendTextDialog(TRequestStatus& aStatus, HBufC*& aText) 
	: iStatus(aStatus), iText(aText)
	{
	}

/**
Destructor for the dialog
*/
CVNCSendTextDialog::~CVNCSendTextDialog()
	{
	}

/**
2nd stage construction of the dialog.
*/
void CVNCSendTextDialog::ConstructL()
	{
	ExecuteLD(R_VNCVIEWER_SEND_TEXT_BASIC_DIALOG);
	}

/**
Handles all commands in the dialog.
Called by the UI framework when a command has been issued.
The command Ids are defined in the .hrh file.

@param aCommandId The command to be executed
@return Whether the dialog can close
*/
TBool CVNCSendTextDialog::OkToExitL(TInt aCommandId)
	{
	TRequestStatus* status = &iStatus;

	switch(aCommandId)
		{
		case EEikBidOk:
			{
			const CEikEdwin* edwin = dynamic_cast<const CEikEdwin*>(ControlOrNull(EVNCViewerSendTextEdwin));	
			iText = edwin->GetTextInHBufL();
			
			User::RequestComplete(status, KErrNone);			
			return ETrue;	
			break;		
			}
		case EEikBidCancel:
			{
			iText = KNullDesC().AllocL();
			User::RequestComplete(status, KErrCancel);			
			return ETrue;			
			break;
			}
		}
		
	return EFalse;
	}

/**
Cancel the dialog
*/
void CVNCSendTextDialog::Cancel()
	{
	TRAPD(err, TryExitL(EVNCViewerSendTextCancel));
	}

/**
Get the capabilities of the dialog

@return TCoeInputCapabilities The capabilites
*/
TCoeInputCapabilities CVNCSendTextDialog::InputCapabilities() const
	{
	return TCoeInputCapabilities::EAllText;
	}



//---------------------------------------------------------------
//---------------------------------------------------------------

/**
Constructor for the active object.

@param aSendText The callback object
*/
CSendTextActive* CSendTextActive::NewL(MSendText &aSendText)
	{
	CSendTextActive* self = new(ELeave) CSendTextActive(aSendText);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
	}

/**
Destrutor for the active object.
*/
CSendTextActive::~CSendTextActive()
	{
	Cancel();
	delete iText;
	}

/**
Constructor for the active object.

@param aSendText The callback object
*/
CSendTextActive::CSendTextActive(MSendText &aSendText)
: CActive(EPriorityStandard), iSendText(aSendText)
	{
	CActiveScheduler::Add(this);
	}

/**
2nd phase constructor
*/
void CSendTextActive::ConstructL()
	{
	// Create the dialog
	iStatus = KRequestPending;
	iDialog = CVNCSendTextDialog::NewL(iStatus, iText);
	SetActive();	
	}
	
/**
The dialog has completed
*/
void CSendTextActive::RunL()
	{
	iSendText.SendTextL(iStatus.Int(), *iText);
	}
	
/**
The dialog needs to be cancelled
*/
void CSendTextActive::DoCancel()
	{
	iDialog->Cancel();	
	iSendText.SendTextL(KErrCancel, KNullDesC());
	}
	
