/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#ifndef VNCVIEWENG_H
#define VNCVIEWENG_H

#include <e32base.h>
#include <W32STD.H>
#include <string.h>
#include <BACLIPB.H>
#include <TXTETEXT.H>
#include <S32MEM.H>
#include <utf.h>
#include <stdio.h>
#include <pthread.h>
#include "keysyms.h"
#include "vncipc.h"

class CVNCMessageWatcher;

// Panic enum
enum TPanic
	{
	EUnexpected,
	ECannotCreateBitmap
	};

/**
Viewer event callback class
*/
class MVNCViewerEvent
	{
public:
	virtual HBufC8* ShowPasswordL() = 0;
	virtual void CreateBitmapL(TInt aHandle) = 0;
	virtual void PutImage(const TRect& aRect) = 0;
	virtual void EngineQuit() = 0;
	virtual void ServerMessageL(const TDesC& aMessage) = 0;
	virtual void MoveSoftCursorL(TInt aX, TInt aY) = 0;
	virtual RWsSprite* CreatePixmapCursorL(TInt aBitmapHandle, TInt aMaskHandle, TInt aHotpointX, TInt aHotpointY) = 0;	
	virtual void WarpPointerL(TInt aCursorX, TInt aCursorY) = 0;
	virtual void FreeCursorL() = 0;
	};
	
class MIpcMsgWaiting
	{
public:
	virtual void MsgWaitingL() = 0;	
	virtual void StdErrWaitingL() = 0;	
	virtual void QuitEngineL() = 0;	
	};
	
/**
Viewer Engine to handle communications to the VNC engine
*/
class CVNCViewMsgProcessor : public CBase, public MIpcMsgWaiting
	{
public:
	static CVNCViewMsgProcessor* NewL(MVNCViewerEvent& aViewerEvent, const TDesC& aCmdParams);
	~CVNCViewMsgProcessor();

	// Send events			
	void SendGraphicsExposeEventL(TInt aX, TInt aY, TInt aWidth, TInt aHeight);
	void SendMouseEventL(const TPointerEvent& aEvent);
	void SendTextL(const TDesC& aText);
	void SendKeyEventL(TInt aType, TUint16 aKeyChar, TInt aModifiers);
	void HandleForegroundEventL(TBool aForeground);
	void RefreshL();
	void PostQuit();
	void SetFullscreen(TBool aOn);
	
	// From MIpcMsgWaiting
	void MsgWaitingL();	
	void StdErrWaitingL();	
	void QuitEngineL();
	
private:
	CVNCViewMsgProcessor(MVNCViewerEvent& aViewerEvent);
	void ConstructL(const TDesC& aCmdParams);
	
	// Process the event queue
	static TInt Event(TAny* aParam);
	void EventL();

	void CreateEngineL(const TDesC& aCmdParams);
	
	void QueueEvent(TInt aMsgType, struct vncmsg& aMsg);
	void ShowPasswordReqL();
	void CreateBitmapL(struct vncmsg& aMsg);
	void PutImageL(struct vncmsg& aMsg);
	void CopyClipboardNowL();
	HBufC8* GetClipboardContentsL();
	void CopyIntoClipboardNowL(struct vncmsg& aMsg);
	void CopyClipboardFromFifoL(TInt aLength);
	void MoveSoftCursorL(struct vncmsg& aMsg);	
	void CreatePixmapCursorL(struct vncmsg& aMsg);	
	void WarpPointerL(struct vncmsg& aMsg);	
	void FreeCursorL(struct vncmsg& aMsg);
	
	void SendKeyEventL(TInt aType, TUint16 aKeyChar);
	void ProcessEventQueueReq();
		
private:
	MVNCViewerEvent&	iViewerEvent;
	CVncIpc*			iVncIpc;
	CVNCMessageWatcher* iMsgWatcher;
	CAsyncCallBack*		iProcessEventQueue;
	RArray<vncmsg>		iEventQueue;
	RFs					iFs;
	int					iEngFd;
	int 				iStdFds[3];
	};

/**
Active object to monitor message activity
*/
class CVNCMessageWatcher : public CActive
	{
public:
	static CVNCMessageWatcher* NewL(CVncIpc& aVncIpc, MIpcMsgWaiting& aMsgWaiting, int aStdErr);
	virtual ~CVNCMessageWatcher();
	
	void Watch();

	static void* WatchThread(void* aArg);
		
private:
	CVNCMessageWatcher(CVncIpc& aVncIpc, MIpcMsgWaiting& aMsgWaiting, int aStdErr);
	void WatchThread();
	void ConstructL();
	void RunL();
	void DoCancel();

private:
	CVncIpc& 		iVncIpc;
	MIpcMsgWaiting& iMsgWaiting;
	int 			iStdErr;
	
	pthread_mutex_t	iMutex;
	pthread_cond_t	iStartCond;
	pthread_cond_t	iCond;
	pthread_t		iWatchThread;
	TThreadId		iThreadId;
	};

	
#endif // VNCVIEWENG_H
