/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#include "VNCViewAP.h"

/**
Populate a list of access points

@param aAccessPoints The list of access points
*/
void CVNCViewAccessPoint::PopulateAccessPointsL(CDesCArray& aAccessPoints)
	{
	// Open a socket
	int sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0) 
	    User::Leave(KErrNoMemory);
	
	// Retrieve the length required
	ifconf ifc;
	memset(&ifc, 0, sizeof(ifconf));
	// Get the length of records
	if (ioctl(sock, SIOCGIFCONF, &ifc))
	  	{
		close(sock);	  	
	    User::Leave(KErrArgument);
  		}

	if (ifc.ifc_len > 0)
		{
		// Allocate the memory
		ifc.ifc_buf = (char*)malloc(ifc.ifc_len);
		if (!ifc.ifc_buf)
			{
			close(sock);	  	
		    User::Leave(KErrNoMemory);
			}
			
		// Read the access points
		if (ioctl(sock, SIOCGIFCONF, &ifc))
		  	{
			free(ifc.ifc_buf);
			close(sock);	  	
		    User::Leave(KErrArgument);
	  		}

		// Now process each access point
		char* ptrConf = ifc.ifc_buf;
		while (ptrConf < ifc.ifc_buf + ifc.ifc_len)
			{
			ifreq* ifr = (ifreq *) ptrConf;
			
			TPtrC8 ptrIfName8((TUint8*)ifr->ifr_name);
			
			// Convert to Unicode
			HBufC* ptrIfName = CnvUtfConverter::ConvertToUnicodeFromUtf8L(ptrIfName8); 
			CleanupStack::PushL(ptrIfName);
			aAccessPoints.AppendL(*ptrIfName);
			CleanupStack::PopAndDestroy(ptrIfName);

			ptrConf += sizeof(ifreq);
			}		
		
		free(ifc.ifc_buf);
		}
		
	close(sock);	
	}
	
