/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * x11.cpp - Wrap commands to the Symbian window system, and provide stubs
 */

// Include files
#include <e32std.h>
#include "x11.h"
#include "x11windows.h"
#include "vncviewer.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Functions which forward on requests to the Symbian Window wrapper 
 */
EXPORT_C Widget XtVaAppInitialize( /* obsolete */
    XtAppContext*	app_context_return,
    const _XtString	/* application_class */,
    XrmOptionDescList	options,
    Cardinal		num_options,
    int*		argc_in_out,
    String*		argv_in_out,
    String*		/* fallback_resources */,
    ...
)
{
	Widget windowgroup = NULL;
	
	// Call the protected version
	TRAPD(err, windowgroup = WindowSystem()->InitAppL(app_context_return, 
		options, num_options, argc_in_out, argv_in_out));
	if (KErrNone != err)
		return 0;
	return windowgroup;	
}

EXPORT_C Display *XtDisplay(
    Widget 		/* widget */
)
{
	return WindowSystem()->GetDisplay();
}

EXPORT_C void XtGetApplicationResources(
    Widget 		/* widget */,
    XtPointer 		base,
    XtResourceList 	resources,
    Cardinal 		num_resources,
    ArgList 		/* args */,
    Cardinal 		/* num_args */
)
{
	// Call the protected version
	TRAPD(err, WindowSystem()->GetAppResourcesL(base, resources, num_resources));
}

EXPORT_C void XtAppAddActions(
    XtAppContext 	/* app_context */,
    XtActionList 	actions,
    Cardinal 		num_actions
)
{
	TRAPD(err, WindowSystem()->AppAddActionsL(actions, num_actions));
}

EXPORT_C XVisualInfo *XGetVisualInfo(
    Display*		/* display */,
    long			vinfo_mask,
    XVisualInfo*	vinfo_template,
    int*			nitems_return
) 
{ 
	XVisualInfo* info = NULL;
	TRAPD(err, info = WindowSystem()->GetVisualInfoL(vinfo_mask, 
		vinfo_template, nitems_return));
   	return info;
}

EXPORT_C XPixmapFormatValues *XListPixmapFormats(
    Display*		/* display */,
    int*		 	count_return 
)
{
	XPixmapFormatValues* formatValues = NULL;
	TRAPD(err, formatValues = WindowSystem()->ListPixmapFormatsL(count_return));
   	return formatValues;
}

EXPORT_C int XFree(
    void*		data
)
{
	WindowSystem()->FreeMemory(data);
	return 0;
}

EXPORT_C void XtVaGetValues(
    Widget		widget,
    ...
)
{
	VA_LIST vaList;
	VA_START(vaList, widget);
	TRAPD(err, WindowSystem()->GetValuesL(widget, vaList));
	VA_END(vaList);
}

EXPORT_C char *XtMalloc(
    Cardinal size
) 
{
	// This will actually leave if there is an error (but leavescan won't like it)
	char* ret = (char*)WindowSystem()->AllocMemoryL(size);
	return ret;
}

EXPORT_C void XtFree(
    char*		ptr 
)
{
	WindowSystem()->FreeMemory(ptr);
}

EXPORT_C Colormap XCreateColormap(
    Display*		/* display */,
    Window		/* w */,
    Visual*		visual,
    int			alloc
)
{
	Colormap colormap = 0;
	TRAPD(err, colormap = WindowSystem()->CreateColormapL(visual, alloc));
	return colormap;
}

EXPORT_C int XQueryColors(
    Display*		/* display */,
    Colormap	colormap,
    XColor*		defs_in_out,
    int			ncolors
)
{
	WindowSystem()->QueryColors(colormap, defs_in_out, ncolors);
	return 1;
}

EXPORT_C Status XAllocColor(
    Display*		/* display */,
    Colormap	colormap,
    XColor*		screen_in_out
)
{
	WindowSystem()->AllocColor(colormap, screen_in_out);
	return 1;
}

EXPORT_C XImage *XCreateImage(
    Display*		/* display */,
    Visual*			visual,
    unsigned int	depth,
    int			/* format */,
    int			/* offset */,
    char*		/* data */,
    unsigned int	width,
    unsigned int	height,
    int			/* bitmap_pad */,
    int			bytes_per_line
)
{
	XImage* image = NULL;
	TRAPD(err, image = WindowSystem()->CreateImageL(visual, depth, width, height, bytes_per_line));
	return image;
}

EXPORT_C XtInputId XtAppAddInput(
    XtAppContext       	/* app_context */,
    int 			source,
    XtPointer 		condition,
    XtInputCallbackProc proc,
    XtPointer 		closure
)
{
	XtInputId id = 0;
	TRAPD(err, id = WindowSystem()->AppAddInputL(source, condition, proc, closure));
	return id;
}

EXPORT_C void XtRemoveInput(
    XtInputId 		id
)
{	
	TRAPD(err, WindowSystem()->AppRemoveInputL(id));
}

EXPORT_C void XtAppProcessEvent(
    XtAppContext 		/* app_context */,
    XtInputMask 		/* mask */
)
{
	// Do not trap the leave, but let it unload the process
	WindowSystem()->AppProcessEventL();
}

EXPORT_C void XtAddEventHandler(
    Widget 		/* widget */,
    EventMask 		event_mask,
    _XtBoolean 		nonmaskable,
    XtEventHandler 	proc,
    XtPointer 		closure
)
{
	TRAPD(err, WindowSystem()->AddEventHandlerL(event_mask, nonmaskable, proc, closure));
}

EXPORT_C int XPutImage(
    Display*		/* display */,
    Drawable		/* d */,
    GC			/* gc */,
    XImage*		/* image */,
    int			src_x,
    int			src_y,
    int			/* dest_x */,
    int			/* dest_y */,
    unsigned int	width,
    unsigned int	height	  
)
{
	TRAPD(err, WindowSystem()->PutImageL(src_x, src_y, width, height));
	return (KErrNone == err) ? 1 : 0;
}

EXPORT_C int XChangeGC(
    Display*		/* display */,
    GC				/* gc */,
    unsigned long	valuemask,
    XGCValues*		values
)
{
	TRAPD(err, WindowSystem()->ChangeGcL(valuemask, values));
	return (KErrNone == err) ? 1 : 0;
}

EXPORT_C int XFillRectangle(
    Display*		/* display */,
    Drawable		/* d */,
    GC			/* gc */,
    int			x,
    int			y,
    unsigned int	width,
    unsigned int	height
)
{
	TRAPD(err, WindowSystem()->FillRectangleL(x, y, width, height));
	return (KErrNone == err) ? 1 : 0;
}

EXPORT_C KeySym XStringToKeysym(
    const char* string
)
{
	return WindowSystem()->Translate(string);
}
 
EXPORT_C void XtGetSelectionValue(
    Widget 		widget,
    Atom 		selection,
    Atom 		target,
    XtSelectionCallbackProc callback,
    XtPointer 		closure,
    XtTime 		xttime
)
{
	WindowSystem()->GetSelectionValue(widget, selection, target, callback, closure, xttime);
}

EXPORT_C int XStoreBytes(
    Display*		/* display */,
    const char*	bytes,
    int			nbytes
)
{
	TRAPD(err, WindowSystem()->StoreBytesL(bytes, nbytes));
	return (KErrNone == err) ? 1 : 0;
}

EXPORT_C Pixmap XCreatePixmap(
    Display*		/* display */,
    Drawable		/* d */,
    unsigned int	width,
    unsigned int	height,
    unsigned int	depth		        
)
{
	Pixmap p = 0;
	TRAPD(err, p = WindowSystem()->CreatePixmapL(width, height, depth));
	return p;
}

EXPORT_C int XCopyArea(
    Display*		/* display */,
    Drawable		src,
    Drawable		dest,
    GC			/* gc */,
    int			src_x,
    int			src_y,
    unsigned int	width,
    unsigned int	height,
    int			dest_x,
    int			dest_y
)
{
	TRAPD(err, WindowSystem()->CopyAreaL(src, dest, src_x, src_y, width, height, dest_x, dest_y));
	return (KErrNone == err) ? 1 : 0;
}

EXPORT_C int XFreePixmap(
    Display*		/* display */,
    Pixmap		pixmap
)
{
	TRAPD(err, WindowSystem()->FreePixmapL(pixmap));
	return (KErrNone == err) ? 1 : 0;
}

EXPORT_C Status XQueryBestCursor(
    Display*		/* display */,
    Drawable		/* d */,
    unsigned int    width,
    unsigned int	height,
    unsigned int*	width_return,
    unsigned int*	height_return
)
{
	*width_return = width;
	*height_return = height;
	return 0;
}

EXPORT_C Pixmap XCreateBitmapFromData(
    Display*		/* display */,
    Drawable		/* d */,
    const char*		data,
    unsigned int	width,
    unsigned int	height
) 
{
	Pixmap p = 0;
	TRAPD(err, p = WindowSystem()->CreateBitmapFromDataL(width, height, data));
	return p;
}

EXPORT_C Cursor XCreatePixmapCursor(
    Display*		/* display */,
    Pixmap		source,
    Pixmap		mask,
    XColor*		/* foreground_color */,
    XColor*		/* background_color */,
    unsigned int	x,
    unsigned int	y			   
)
{
	Cursor c;
	TRAPD(err, c = WindowSystem()->CreatePixmapCursorL(source, mask, x, y));
	return c;
}

EXPORT_C int XWarpPointer(
    Display*		/* display */,
    Window		/* src_w */,
    Window		/* dest_w */,
    int			/* src_x */,
    int			/* src_y */,
    unsigned int	/* src_width */,
    unsigned int	/* src_height */,
    int			dest_x,
    int			dest_y	     
)
{
	TRAPD(err, WindowSystem()->WarpPointerL(dest_x, dest_y));
	return (KErrNone == err) ? 1 : 0;
}

EXPORT_C int XFreeCursor(
    Display*		/* display */,
    Cursor		cursor
)
{
	TRAPD(err, WindowSystem()->FreeCursorL(cursor));
	return (KErrNone == err) ? 1 : 0;
}

/*
 * Dummy functions which allow the program to run
 */
EXPORT_C GC XCreateGC(
    Display*		/* display */,
    Drawable		/* d */,
    unsigned long	/* valuemask */,
    XGCValues*		/* values */
) 
{
	return GC();
}

EXPORT_C void XtVaSetValues(
    Widget		/* widget */,
    ...
)
{
}

EXPORT_C int XInstallColormap(
    Display*		/* display */,
    Colormap		/* colormap */
)
{
	// Ignore for now
	return 1;
}

EXPORT_C int XFreeColors(
    Display*		/* display */,
    Colormap		/* colormap */,
    unsigned long*	/* pixels */,
    int			/* npixels */,
    unsigned long	/* planes */
)
{
	// Do nothing
	return 1;
}

EXPORT_C Widget XtVaCreateManagedWidget(
    const _XtString	/* name */,
    WidgetClass		/* widget_class */,
    Widget			/* parent */,
    ...
)
{
	return 0;
}


EXPORT_C char *XawDialogGetValueString
(
	 Widget			/* w */
)
{
	return NULL;
}

EXPORT_C String XtNewString(String /* str */) 
{
	return NULL;
}

EXPORT_C char *XDisplayName(
    const char*	/* string */
)
{
	return NULL;
}

EXPORT_C Display *XOpenDisplay(
    const char*	/* display_name */
)
{
	return NULL;
}

EXPORT_C char **XListFonts(
    Display*		/* display */,
    char*		/* pattern */,
    int			/* maxnames */,
    int*		/* actual_count_return */
)
{
	return NULL;
}

EXPORT_C XErrorHandler XSetErrorHandler (
    XErrorHandler	handler
)
{
	return handler;
}

EXPORT_C XIOErrorHandler XSetIOErrorHandler (
    XIOErrorHandler	handler
) 
{
	return handler;
}

EXPORT_C XtErrorHandler XtAppSetErrorHandler(
    XtAppContext 	/* app_context */,
    XtErrorHandler  handler 
) 
{
	return handler;
}

EXPORT_C char *XFetchBytes(
    Display*		/* display */,
    int*		/* nbytes_return */
)
{
	return NULL;
}

EXPORT_C XImage *XShmCreateImage(
    Display*		/* dpy */,
    Visual*		/* visual */,
    unsigned int	/* depth */,
    int			/* format */,
    char*		/* data */,
    XShmSegmentInfo*	/* shminfo */,
    unsigned int	/* width */,
    unsigned int	/* height */
)
{
	return NULL;
}


EXPORT_C void XtAddActions( /* obsolete */
    XtActionList 	/* actions */,
    Cardinal 		/* num_actions */
)
{
}

EXPORT_C int XDefaultDepth(
    Display*		/* display */,
    int			/* screen_number */
)
{
	return 0;
}

EXPORT_C int XDefineCursor(
    Display*		/* display */,
    Window		/* w */,
    Cursor		/* cursor */
)
{
	return 0;
}

EXPORT_C Widget XtWindowToWidget(
    Display*		/* display */,
    Window 		/* window */
)
{
	return Widget();
}

EXPORT_C void XmuCvtStringToBackingStore
(
 XrmValue*			/* args */,
 Cardinal*			/* num_args */,
 XrmValuePtr		/* fromVal */,
 XrmValuePtr		/* toVal */
 )
{
}
 
EXPORT_C void XtAddConverter( /* obsolete */
    const _XtString	/* from_type */,
    const _XtString 	/* to_type */,
    XtConverter 	/* converter */,
    XtConvertArgList 	/* convert_args */,
    Cardinal 		/* num_args */
)
{
}
 
EXPORT_C void XtVaGetApplicationResources(
    Widget		/* widget */,
    XtPointer		/* base */,
    XtResourceList	/* resources */,
    Cardinal		/* num_resources */,
    ...
) 
{
}

EXPORT_C int XChangeWindowAttributes(
    Display*		/* display */,
    Window		/* w */,
    unsigned long	/* valuemask */,
    XSetWindowAttributes* /* attributes */
)
{
	return 0;
}

EXPORT_C Window XtWindowOfObject(
    Widget 		/* object */
)
{
	return Window();
}

EXPORT_C int XSetInputFocus(
    Display*	/* display */,
    Window		/* focus */,
    int			/* revert_to */,
    XtTime		/* time */
)
{
	return 0;
}

EXPORT_C KeySym XKeycodeToKeysym(
    Display*		/* display */,
    unsigned int	/* keycode */,
    int				/* index */
)
{
	return KeySym();
}

EXPORT_C Bool XCheckTypedWindowEvent(
    Display*	/* display */,
    Window		/* w */,
    int			/* event_type */,
    XEvent*		/* event_return */
)
{
	return False;
}

EXPORT_C int XLookupString(
    XKeyEvent*	/* event_struct */,
    char*		/* buffer_return */,
    int			/* bytes_buffer */,
    KeySym*		/* keysym_return */,
    XComposeStatus*	/* status_in_out */
)
{
	return 0;
}

EXPORT_C Bool IsModifierKey(
	KeySym		/* keysym */
)
{
	return True;
}

EXPORT_C Status XAllocNamedColor(
    Display*	/* display */,
    Colormap	/* colormap */,
    const char*	/* color_name */,
    XColor*		/* screen_def_return */,
    XColor*		/* exact_def_return */
)
{
	return 0;
}
 
EXPORT_C int XSync(
    Display*	/* display */,
    Bool		/* discard */
)
{
	return 0;
}

EXPORT_C Widget XtVaCreatePopupShell(
    const _XtString	/* name */,
    WidgetClass		/* widgetClass */,
    Widget			/* parent */,
    ...
)
{
	return Widget();
}
 
EXPORT_C Screen *XtScreen(
    Widget 		/* widget */
) 
{
	return NULL;
}

EXPORT_C void XtMoveWidget(
    Widget 			/* widget */,
    _XtPosition		/* x */,
    _XtPosition		/* y */
)
{
}

EXPORT_C void XtPopup(
    Widget 			/* popup_shell */,
    XtGrabKind 		/* grab_kind */
)
{
}

EXPORT_C void XtRealizeWidget(
    Widget 			/* widget */
)
{
}
 
EXPORT_C void XtPopdown(
    Widget 			/* popup_shell */
)
{
}

EXPORT_C Widget XtNameToWidget(
    Widget 			/* reference */,
    const _XtString	/* names */
)
{
	return Widget();
} 

EXPORT_C int XReparentWindow(
    Display*		/* display */,
    Window			/* w */,
    Window			/* parent */,
    int				/* x */,
    int				/* y */
)
{
	return NULL;
}

EXPORT_C void XtResizeWidget(
    Widget 			/* widget */,
    _XtDimension	/* width */,
    _XtDimension	/* height */,
    _XtDimension	/* border_width */
)
{
}

EXPORT_C void XtUnmanageChild(
    Widget 			/* child */
)
{
}

EXPORT_C void XtManageChild(
    Widget 			/* child */
)
{
}

EXPORT_C int XtGrabKeyboard(
    Widget 			/* widget */,
    _XtBoolean 		/* owner_events */,
    int 			/* pointer_mode */,
    int 			/* keyboard_mode */,
    XtTime 			/* time */
)
{
	return 0;
}

EXPORT_C void XtUngrabKeyboard(
    Widget 			/* widget */,
    XtTime 			/* time */
)
{
}

EXPORT_C void XtMapWidget(Widget /* w */) 
{
}

EXPORT_C void XtUnmapWidget(Widget /* w */)
{
}

EXPORT_C void XtRemoveTimeOut(
    XtIntervalId 	/* timer */
)
{
}

EXPORT_C void XawViewportSetCoordinates
(
 Widget				/* gw */,
 Position			/* x */,
 Position			/* y */
)
{
}

EXPORT_C XtIntervalId XtAppAddTimeOut(
    XtAppContext 	/* app_context */,
    unsigned long 	/* interval */,
    XtTimerCallbackProc /* proc */,
    XtPointer 		/* closure */
)
{
	return 0;
}

EXPORT_C Bool XCheckIfEvent(
    Display*		/* display */,
    XEvent*			/* event_return */,
    Bool (*) (
	       Display*		/* display */,
               XEvent*	/* event */,
               XPointer	/* arg */
             )		/* predicate */,
    XPointer		/* arg */
)
{
	return True;
}

EXPORT_C int XCloseDisplay(
    Display*		/* display */
)
{
	return 0;
}

EXPORT_C int XFreeFontNames(
    char**			/* list */
)
{
	return 0;
}

EXPORT_C Font XLoadFont(
    Display*		/* display */,
    const char*		/* name */
)
{
	return Font();
}

EXPORT_C int XBell(
    Display*		/* display */,
    int				/* percent */
)
{
	return 0;
}

EXPORT_C int XForceScreenSaver(
    Display*		/* display */,
    int				/* mode */
)
{
	return 0;
}

EXPORT_C Window XCreateWindow(
    Display*		/* display */,
    Window			/* parent */,
    int				/* x */,
    int				/* y */,
    unsigned int	/* width */,
    unsigned int	/* height */,
    unsigned int	/* border_width */,
    int				/* depth */,
    unsigned int	/* class */,
    Visual*			/* visual */,
    unsigned long	/* valuemask */,
    XSetWindowAttributes*	/* attributes */
)
{
	return Window();
} 

EXPORT_C int XMapWindow(
    Display*		/* display */,
    Window			/* w */
)
{
	return 0;
}

EXPORT_C int XDrawString(
    Display*		/* display */,
    Drawable		/* d */,
    GC				/* gc */,
    int				/* x */,
    int				/* y */,
    const char*		/* string */,
    int				/* length */
) 
{
	return 0;
}

EXPORT_C int XFlush(
    Display*		/* display */
)
{
	return 0;
}

EXPORT_C int XDestroyWindow(
    Display*		/* display */,
    Window			/* w */
)
{
	return 0;
}

EXPORT_C int XGrabKeyboard(
    Display*		/* display */,
    Window			/* grab_window */,
    Bool			/* owner_events */,
    int				/* pointer_mode */,
    int				/* keyboard_mode */,
    XtTime			/* time */
)
{
	return 0;
}

EXPORT_C int XUngrabKeyboard(
    Display*		/* display */,
    XtTime			/* time */
)
{
	return 0;
}

EXPORT_C Atom XInternAtom(
    Display*		/* display */,
    const char*		/* atom_name */,
    Bool			/* only_if_exists */		 
)
{
	return Atom();
}

EXPORT_C Status XSetWMProtocols(
    Display*		/* display */,
    Window			/* w */,
    Atom*			/* protocols */,
    int				/* count */
)
{
	return 0;
}

EXPORT_C XtTranslations XtParseTranslationTable(
    const _XtString	/* table */
) 
{
	return XtTranslations();
}

EXPORT_C Window XtWindow(
    Widget 			/* widget */
)
{
	return Window();
}

EXPORT_C void XtOverrideTranslations(
    Widget 			/* widget */,
    XtTranslations 	/* translations */
)
{
}

EXPORT_C Status XFetchName(
    Display*		/* display */,
    Window			/* w */,
    char**			/* window_name_return */
)
{
	return 0;
}

EXPORT_C Status XIconifyWindow(
    Display*		/* display */,
    Window			/* w */,
    int				/* screen_number */
)
{
	return 0;	
}

EXPORT_C int XGetWindowProperty(
    Display*		/* display */,
    Window			/* w */,
    Atom			/* property */,
    long			/* long_offset */,
    long			/* long_length */,
    Bool			/* delete */,
    Atom			/* req_type */,
    Atom*			/* actual_type_return */,
    int*			/* actual_format_return */,
    unsigned long*	/* nitems_return */,
    unsigned long*	/* bytes_after_return */,
    unsigned char**	/* prop_return */
)
{
	return 0;	
}

EXPORT_C Status XQueryTree(
    Display*		/* display */,
    Window			/* w */,
    Window*			/* root_return */,
    Window*			/* parent_return */,
    Window**		/* children_return */,
    unsigned int*	/* nchildren_return */
)
{
	return 0;
}

EXPORT_C void XtVaGetSubresources(
    Widget			/* widget */,
    XtPointer		/* base */,
    const _XtString	/* name */,
    const _XtString	/* class */,
    XtResourceList	/* resources */,
    Cardinal		/* num_resources */,
    ...
)
{	
}

EXPORT_C int XStoreColor(
    Display*		/* display */,
    Colormap		/* colormap */,
    XColor*			/* color */
)
{
	return 0;
}

EXPORT_C int XMapRaised(
    Display*		/* display */,
    Window			/* w */
)
{
	return 0;
}

EXPORT_C int XSelectInput(
    Display*		/* display */,
    Window			/* w */,
    long			/* event_mask */
)
{
	return 0;
}

EXPORT_C void XtAddRawEventHandler(
    Widget 			/* widget */,
    EventMask 		/* event_mask */,
    _XtBoolean 		/* nonmaskable */,
    XtEventHandler 	/* proc */,
    XtPointer 		/* closure */
)
{
}

EXPORT_C Boolean XmuConvertStandardSelection
(
 Widget				/* w */,
 XtTime				/* timev */,
 Atom*				/* selection */,
 Atom*				/* target */,
 Atom*				/* type_return */,
 XPointer*			/* value_return */,
 unsigned long*		/* length_return */,
 int*				/* format_return */
 )
{
	return True;
}

EXPORT_C Boolean XtOwnSelection(
    Widget 			/* widget */,
    Atom 			/* selection */,
    XtTime 			/* time */,
    XtConvertSelectionProc /* convert */,
    XtLoseSelectionProc	/* lose */,
    XtSelectionDoneProc /* done */
)
{
	return True;
}

EXPORT_C void XtRegisterDrawable(
    Display*		/* dpy */,
    Drawable		/* drawable */,
    Widget			/* widget */
)
{
}

EXPORT_C void* signal(int /* signum */, sig_act /* action */)
{
	return NULL;	
}

EXPORT_C int XDestroyImage(XImage* /* ximage */)
{
	return 0;	
}

EXPORT_C Status XShmAttach(
    Display*			/* dpy */,
    XShmSegmentInfo*	/* shminfo */
)
{
	return 0;
}

EXPORT_C Bool XShmQueryExtension(
    Display*		/* dpy */
)
{
	return FALSE;
}



/*
 * New functions under the X11 umbrella, but not actually standard X11 calls
 */
EXPORT_C int XtInit()
{
	return (WindowSystem() == NULL) ? 0 : 1;
}

EXPORT_C void XtUnInit()
{
	DestroyWindowSystem();
}

EXPORT_C char* Xtgetpass(
    const char* prompt
)
{
	char* password = NULL;
	TRAPD(err, password = WindowSystem()->GetPassL(prompt));
	return password;	
}

EXPORT_C void XtPrepareMemoryTransfer()
{
	WindowSystem()->PrepareMemoryTransfer();
}

EXPORT_C void XtEndMemoryTransfer()
{
	WindowSystem()->EndMemoryTransfer();	
}

EXPORT_C char* XtMemoryBuffer()
{
	return WindowSystem()->MemoryBuffer();
}

EXPORT_C int JpegConvert(Visual* vis, int width, int height, char* data, long size)
{
	TRAPD(err, WindowSystem()->JpegConvertL(vis, width, height, data, size));
	return (KErrNone == err) ? 1 : 0;	
}

EXPORT_C char* JpegGetData()
{
	char* data = NULL;
	TRAPD(err, data = WindowSystem()->JpegGetDataL());
	return data;
}

EXPORT_C void JpegDiscardData()
{
	WindowSystem()->JpegDiscardData();
}

EXPORT_C Pixmap XCreateBitmapWithData(int width, int height, int visdepth, unsigned char* data)
{
	Pixmap p = 0;
	TRAPD(err, p = WindowSystem()->CreateBitmapWithDataL(width, height, visdepth, data));
	return p;
}

EXPORT_C int XDrawBitmap(Pixmap bitmap, Pixmap mask, int x, int y, int width, int height, int hotpointx, int hotpointy)
{
	TRAPD(err, WindowSystem()->DrawBitmapL(bitmap, mask, x, y, width, height, hotpointx, hotpointy));
	return (KErrNone == err) ? 1 : 0;	
}

#ifdef __cplusplus
}
#endif
