/*
 *  Copyright (C) 2007 Rob Cliff.  All Rights Reserved.
 *  Copyright (C) 2007 Savage Minds Ltd.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

/*
 * x11Parsers.cpp - Parse the options
 */

#include "x11Parsers.h"

/**
Create the option parse
@return COptionParser* The option parser
*/
COptionParser* COptionParser::NewL()
	{
	COptionParser* parser = new(ELeave) COptionParser();
	CleanupStack::PushL(parser);
	parser->ConstructL();
	CleanupStack::Pop(parser);
	return parser; 
	}
	
/**
Destructor
*/
COptionParser::~COptionParser()
	{
	iOptions.ResetAndDestroy();
	}
	
/**
Constructor
*/
COptionParser::COptionParser()
: iLastArgKind(XrmoptionEmpty)
	{
	}
	
/**
2nd phase constructor
*/
void COptionParser::ConstructL()
	{
	}

/**
Parse an argument
@param aNumOptions The number of available options
@param aOptionRecords The option records
@param aOption The command option to parse
*/
TBool COptionParser::ParseArgumentL(Cardinal aNumOptions, const XrmOptionDescList aOptionRecords, const TDesC8& aOption)
	{
	if ((iLastArgKind == XrmoptionSepArg) && (iOptions.Count() > 0))
		{
		// The last argument had a value
		CNVPairDes* option = iOptions[iOptions.Count() - 1];
		option->SetValueL(aOption);
		iLastArgKind = XrmoptionEmpty;
		return ETrue;
		}
	else
		{
		for(TInt count = 0; count < aNumOptions; ++count)
			{
			// Find the option
			const XrmOptionDescRec& rec = aOptionRecords[count];
			
			TPtrC8 ptrOption((TUint8*)rec.option);
			if (ptrOption.CompareF(aOption) == 0)
				{
				CNVPairDes* option = NULL;
				
				TPtrC8 ptrResName((TUint8*)rec.specifier);
				
				iLastArgKind = rec.argKind;
				if (iLastArgKind == XrmoptionNoArg)
					{
					TPtrC8 ptrValue((TUint8*)rec.value);
					option = CNVPairDes::NewL(ptrResName.Mid(1), ptrValue);
					}
				else
					option = CNVPairDes::NewL(ptrResName.Mid(1));
					
				iOptions.Append(option);
				
				return ETrue;
				}
			}
		}
	return EFalse;
	}

/**
Update the data structure with a resource
@param aResource The resource 
@param aPtr The data structure
*/
void COptionParser::UpdateResourceL(const XtResource& aResource, void* aPtr)
	{
	TPtrC8 ptrResName((TUint8*)aResource.resource_name);
	for(TInt count = 0; count < iOptions.Count(); ++count)
		{
		// Find the appropriate resource
		CNVPairDes* option = iOptions[count];		
		if (option->Name().CompareF(ptrResName) == 0)
			{
			// Check the type of the argument
			TPtrC8 ptrArgType((TUint8*)aResource.resource_type);
			if (ptrArgType.CompareF(TPtrC8((TUint8*)XtRBool)) == 0)
				{
				Bool value = False;
				if (option->Value().CompareF(_L8("True")) == 0)
					{
					value = True;
					}
				
				Mem::Copy(aPtr, (void*)&value, aResource.resource_size);	
				}
			if (ptrArgType.CompareF(TPtrC8((TUint8*)XtRString)) == 0)
				{
				TPtrC8 ptrValue(option->Value());
				char* tempPtr = (char*)User::Alloc(ptrValue.Length() + 1);
				Mem::FillZ(tempPtr, ptrValue.Length() + 1);
				Mem::Copy(tempPtr, ptrValue.Ptr(), ptrValue.Length());	
				Mem::Copy(aPtr, (void*)&tempPtr, aResource.resource_size);	
				}
			if (ptrArgType.CompareF(TPtrC8((TUint8*)XtRInt)) == 0)
				{
				TLex8 lex(option->Value());
				TInt val = 0;
				if (KErrNone == lex.Val(val))
					{
					Mem::Copy(aPtr, &val, aResource.resource_size);	
					}
				}
			}
		}
	}



